import * as React from 'react';
import {observer} from "mobx-react-lite";
import {DashboardComponentModel} from "../../../../stores/DashboardComponentStore";
import {HintButton, HintButtonProps, InputField, SelectField, useReadonlyContext} from "@rx-suite/ui";
import {Parameter} from "../../../../stores/Types";
import {t} from "@lingui/macro";
import {makeStyles} from "@material-ui/core/styles";

export type TimeLengthParameterFieldProps = {
    component: DashboardComponentModel,
    parameter: Parameter,
    hint: HintButtonProps
};

const useStyles = makeStyles(theme => ({
    wrapper: {
        display: "flex",
        width: '100%',
        alignItems: 'flex-end',
        '& > *': {
            marginRight: theme.spacing(1)
        }
    }
}));

export const TimeLengthParameterField: React.FunctionComponent<TimeLengthParameterFieldProps> = observer(({
                                                                                                              component,
                                                                                                              parameter,
                                                                                                              hint
                                                                                                          }) => {
    const classes = useStyles();
    const readOnly = useReadonlyContext();
    const [lengthUnit, setLengthUnit] = React.useState<'second' | 'minute' | 'hour' | 'day'>('hour');
    const handleChange = React.useCallback(e => {
        let resolvedValue = null
        if (e.target.value !== '') {
            const passedValue = Number(e.target.value)
            switch (lengthUnit) {
                case "day":
                    resolvedValue = passedValue * (60 * 60 * 24);
                    break;
                case "hour":
                    resolvedValue = passedValue * (60 * 60);
                    break;
                case "minute":
                    resolvedValue = passedValue * 60;
                    break;
                case "second":
                default:
                    resolvedValue = passedValue;
                    break;
            }
        }
        component.nestedAssignment(e.target.name, resolvedValue)
    }, [component, lengthUnit]);
    const name = `parameters.${parameter.id}`;
    const rawValue = component.viewModel.parameters[parameter.id];
    const value = Number(rawValue);
    let convertedValue;
    switch (lengthUnit) {
        case "day":
            convertedValue = value / (60 * 60 * 24);
            break;
        case "hour":
            convertedValue = value / (60 * 60);
            break;
        case "minute":
            convertedValue = value / 60;
            break;
        case "second":
        default:
            convertedValue = value;
            break;
    }
    const plural = (convertedValue !== 1);
    const inputProps = {step: "any"} as { step: any, min?: number, max?: number };
    if (parameter.min || (parameter.min === 0)) {
        inputProps.min = parameter.min;
    }
    if (parameter.max || (parameter.max === 0)) {
        inputProps.max = parameter.max;
    }
    return (
        <div className={classes.wrapper}>
            <InputField label={parameter.name} type={'number'}
                        value={(isNaN(convertedValue) || ['', null, undefined].includes(rawValue)) ? '' : convertedValue.toString()}
                        inputProps={inputProps}
                        name={name}
                        onChange={handleChange}
                        disabled={readOnly}
            />
            <SelectField
                onChange={(_name, value) => {
                    setLengthUnit(value);
                }}
                value={lengthUnit}
                options={[
                    {label: plural ? t`Seconds` : t`Second`, value: 'second'},
                    {label: plural ? t`Minutes` : t`Minute`, value: 'minute'},
                    {label: plural ? t`Hours` : t`Hour`, value: 'hour'},
                    {label: plural ? t`Days` : t`Day`, value: 'day'},
                ]}
                disabled={readOnly}
            />
            <HintButton {...hint}/>
        </div>
    );
});