import {attribute, DataStore, model, Model, Types} from '@rx-suite/data';
import {t} from "@lingui/macro";
import * as yup from 'yup';
import {computed} from "mobx";
import {Parameter} from "./Types";
import { computedFn } from 'mobx-utils';

@model({
    transportLayer: "/api/dashboard_targets",
    routingInfo: {}
})
export class DashboardTargetModel extends Model {
    types: Types = {
        ui_hints: 'json',
        parameters: 'json',
        returns: 'json',
    };

    constructor() {
        super();
        super.setup();
    }

    @attribute({
        validation: yup.string()
    })
    module_id = '';

    @attribute({
        label: t`Module`,
        validation: yup.string()
    })
    module_name = '';

    @attribute({
        label: t`ID`,
        validation: yup.string()
    })
    dashboard_id = '';

    @attribute({
        label: t`Name`,
        validation: yup.string()
    })
    name = '';

    @attribute({
        label: t`Description`,
        validation: yup.string()
    })
    desc = '';

    @attribute({
        validation: yup.mixed()
    })
    ui_hints: any = {};

    @attribute({
        validation: yup.mixed()
    })
    parameters: { [s: string]: Parameter } = {};

    @attribute({
        validation: yup.mixed()
    })
    returns: any = null;

    @attribute({label: t`Created At`, autoForm: false})
    created_at: Date = new Date();

    @attribute({label: t`Updated At`, autoForm: false})
    updated_at: Date = new Date();

    @computed
    get displayTypes() {
        if (this.ui_hints && this.ui_hints.type) {
            switch (this.ui_hints.type) {
                case 'category_chart':
                    return [
                        {label: t`Bar`, value: 'bar'},
                        {label: t`Pie`, value: 'pie'},
                        {label: t`Radar`, value: 'radar'},
                        {label: t`Polar Area`, value: 'polararea'},
                        (this.ui_hints.target_type !== 'series') && {label: t`Counter`, value: 'counter'},
                        {label: t`Tree Map`, value: 'treemap'},
                    ].filter(t => t);
                case 'dual_axis_chart':
                    return [
                        {label: t`Bar`, value: 'bar'},
                        {label: t`Line`, value: 'line'},
                        {label: t`Scatter`, value: 'scatter'},
                        {label: t`Heat Map`, value: 'heatmap'},
                    ];
                case 'table':
                    return [{label: t`Table`, value: 'table'}];
                case 'gauge':
                    return [
                        {label: t`Gauge`, value: 'gauge'},
                        {label: t`Counter`, value: 'counter'}
                    ];
                case 'score_card':
                    return [
                        {label: t`Score Card`, value: 'score_card'}
                    ]
                case 'lab_status':
                    return [
                        {label: t`Lab Status`, value: 'lab_status'}
                    ]
                default:
                    return [];
            }
        }
        return [];
    }

    @computed
    get isScoreCard() {
        return Object.values(this.parameters).some((parameterDef: Parameter) => parameterDef.single_row_parameter)
    }

    columnDefaultWidth = computedFn(function getWidth(this: DashboardTargetModel, columnId: string): any {
        const displayOpts = this?.ui_hints?.column_display_props;
        if (displayOpts && displayOpts[columnId]){
            return displayOpts[columnId].width;
        }
        return undefined;
    })

    isColumnFilterable = computedFn(function getFilterable(this: DashboardTargetModel, columnId: string): any {
        const displayOpts = this?.ui_hints?.column_display_props;
        if (displayOpts && displayOpts[columnId]){
            return displayOpts[columnId].filterable;
        }
        return true;
    })
}

export const DashboardTargetStore = DashboardTargetModel.store as unknown as DataStore<DashboardTargetModel>;
DashboardTargetStore.setFetchParams({limit: undefined, page: 0, sorted: [], filtered: []});
