import {attribute, DataStore, model, Model} from '@rx-suite/data';
import {t} from "@lingui/macro";
import * as yup from 'yup';
import {computed, observable, override} from "mobx";
import React from "react";

@model({})

export class LabLayoutComponentModel extends Model {
    constructor() {
        super();
        super.setup();
    }

    @attribute({
        label: t`id`,
        validation: yup.number()
    })
    id = -1;

    @attribute({
        label: t`Name`,
        validation: yup.string()
    })
    name = "";

    @attribute({
        label: t`x`,
        validation: yup.number()
    })
    x = 0;

    @attribute({
        label: t`y`,
        validation: yup.number()
    })
    y = 0;

    @attribute({
        label: t`z index`,
        validation: yup.number()
    })
    z_index = 0;

    @attribute({
        label: t`Width`,
        validation: yup.number()
    })
    width = 120;

    @attribute({
        label: t`Height`,
        validation: yup.number()
    })
    height = 120;

    @attribute({
        label: t`aspect_ratio`,
        validation: yup.number()
    })
    aspect_ratio = 1;

    @attribute({
        label: t`view type`,
        validation: yup.string()
    })
    component_type: LayoutComponentType = "machine_status_dot";

    // Machine target used for a status dot component
    @attribute({
        label: t`machine sn`,
        validation: yup.string()
    })
    machine_sn = "";

    // Machine groups used for an alert component
    @attribute({
        label: t`Machine Groups to Monitor`
    })
    machine_group_ids: number[] = [];

    @attribute({
        validation: yup.string().nullable()
    })
    image_url: string | null = null;

    @attribute({
        label: t`Visual Alert After`,
        validation: yup.number().positive()
    })
    alert_after = 300;

    @observable file: Blob | undefined;
    @observable image: CanvasImageSource | undefined;

    @computed
    get position(): LayoutComponentPosition {
        return {x: this.x, y: this.y};
    }

    @computed
    get isMachineComponent(): boolean {
        return this.component_type === "machine_status_dot";
    }

    @computed
    get isAlertGroupComponent(): boolean {
        return this.component_type === "alert_group";
    }

    @override
    handleChange(e: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>) {
        let value;
        if(Object.prototype.hasOwnProperty.call(e.target, 'checked')) {
            value = (e.target as HTMLInputElement).checked;
        } else {
            if (isNaN((e.target.value as unknown) as number)) {
                value = e.target.value;
            } else {
                // To work with the canvas, we need the model value to reflect a number
                value = Number(e.target.value);
            }
        }
        this.nestedAssignment(e.target.name, value);
    }

    pointOverlapsComponentCorner(point: LayoutComponentPosition): boolean {
        return Math.abs(this.x - point.x) < this.width &&
            Math.abs(this.x - point.x) > (this.width - 15) &&
            Math.abs(this.y - point.y) < this.height &&
            Math.abs(this.y - point.y) > (this.height - 15)
    }

    pointOverlapsComponent(point: LayoutComponentPosition): boolean {
        return point.x > this.x &&
            point.x < (this.x + this.width) &&
            point.y > this.y &&
            point.y < (this.y + this.height)
    }

}

export type LayoutComponentType = "machine_status_dot" | "image" | "alert_group";

export interface LayoutComponentPosition {
    x: number;
    y: number;
}

export const LabLayoutComponentStore = LabLayoutComponentModel.store as DataStore<LabLayoutComponentModel>;

